# Positively Split Ewald, version 3 (PSEv3)
PSEv3 is a HOOMD plugin by Andrew M. Fiore containing a GPU implementation of the Fast Stokesian Dynamics
(FSD) algorithm for calculation of the Stokesian Dynamics hydrodynamic resistance tensor and associated
stochastic thermal forces and displacements.

The theory behind the FSD method is described in detail in the following reference:

1. **Fast Stokesian Dynamics**, Andrew M. Fiore and James W. Swan, Journal of Fluid Mechanics, **XXX**,
YYY (2019).[DOI](http://doi.org/10.1063/1.4978242) [arXiv](https://arxiv.org/abs/1611.09322)

Other references that may be useful are

2. **Rapid Sampling of Stochastic Displacements in Brownian Dynamics
Simulations**, Andrew M. Fiore, Florencio Balboa Usabiaga, Aleksandar
Donev, and James W. Swan, The Journal of Chemical Physics, **146**,
124116 (2017).[DOI](http://doi.org/10.1063/1.4978242) [arXiv](https://arxiv.org/abs/1611.09322)

3. **Rapid Sampling of Stochastic Displacements in Brownian Dynamics
Simulations with Stresslet Constraints**, Andrew M. Fiore and James W. Swan, The Journal of Chemical Physics, **148**,
063302 (2017).[DOI](https://doi.org/10.1063/1.5005887) [arXiv](https://arxiv.org/abs/1709.06946)

To view the documentation, clone the repository and open the [index file](docs/_build/html/index.html) with a web browser.

## Software Requirements

The FSD plugin requires the following software:
 - HOOMD, compiled with CUDA (tested with version 2.4)
 - CUDA (tested with version 9.2)
 - LAPACKE (tested with version 3.6.1)
 - CBLAS (tested with version 3.6.1)
 - CUSP (cloned from [github](https://github.com/cusplibrary/cusplibrary.git), there is an older version online that can be found but is not compatible with newer versions of CUDA. Be sure to use this github repository specifically.)

CUDA can be installed manually after downloading the source code from 
[NVIDIA](https://docs.nvidia.com/cuda/cuda-installation-guide-linux/index.html) 
and following the instructions presented by them.

LAPACKE and CBLAS can be installed manually after downloading the source code from 
[netlib](http://www.netlib.org/lapack/lapacke.html) and [openblas](https://www.openblas.net/) 
or from repositories. In Ubuntu, the simplest method is via repository:
```
	sudo apt-get install liblapack3 liblapack-dev liblapacke liblapacke-dev
	sudo apt-get install libblas3 libblas-dev libopenblas-dev libatlas-base-dev
```

## Installing HOOMD-blue

For use with plugins, HOOMD-blue needs to be compiled from source (not installed using anaconda packages). 
The [documentation](https://hoomd-blue.readthedocs.io/en/stable/compiling.html) for HOOMD-blue provides 
directions on installation. Those directions can be followed with the exception that the cmake command 
needs a particular set of flags, i.e. 

```
	export HOOMD_ROOT=/path/to/prefix
	git clone --recursive https://bitbucket.org/glotzer/hoomd-blue
	mkdir build
	cd build
	cmake ../ -DCMAKE_INSTALL_PREFIX=${HOOMD_ROOT}/lib/python -DCOPY_HEADERS=ON -DENABLE_CUDA=ON -DSINGLE_PRECISION=ON
	make -j6
	make install
```

A sample installation location is
```
	export HOOMD_ROOT=/home/your_user_name/hoomd-blue-install
```
An alternative to the cmake command for setting the flags is:
```
	cmake ../
	ccmake .
```
with the following ccmake options
```
	CMAKE_INSTALL_PREFIX 	/path/to/prefix
	COPY_HEADERS		ON
	ENABLE_CUDA		ON
	SINGLE_PRECISION	ON
```
after which you can hit "c" to configure cmake and "g" to generate the configuration. The installation can be tested with::

	make test

Finally, HOOMD has to be added to the python path by adding the following command to the user's
`.bashrc` file
```
	export PYTHONPATH=$PYTHONPATH:${HOOMD_ROOT}/lib/python
```

## Installing the FSD Plugin (PSEv3)

The FSD plugin itself can be installed using the following commands from the plugin directory
```
	make plugin_build
	cd plugin_build
	cmake ../ -DHOOMD_ROOT=${HOOMD_ROOT}/lib/python/hoomd
	make -j6
	make install
```
By default, `make install` will install the plugin into
```
	${HOOMD_ROOT}/lib/python/hoomd/PSEv3
```

## Using the Plugin

A detailed description of a python script to set up and ran a simulation is given in `examples/run.py`.
You can call this script with the command
```
	python3 run.py
```
